#ifndef SERIALCONSOLE_H
#define SERIALCONSOLE_H

#include <QDockWidget>
#include <QtSerialPort/QSerialPort>
#include <QQueue>
#include <QTime>

class SimpleTextEdit;
class QScrollBar;
class QLineEdit;
class QTimer;

class SerialConsole : public QDockWidget
{
    Q_OBJECT
public:
    explicit SerialConsole(QWidget *parent = 0);

signals:
    void updateTemperature(double,double);
    void updateCoordinateX(double);
    void updateCoordinateY(double);
    void updateCoordinateZ(double);
    void updateWorkProgress(int);
    void updateETA(double);

private slots:
    void readMessage();
    void sendMessage();
    void requestTemperature();
    void kickAfterTimeout();
    void connectTo(const QString &portname);
    void updateScrollBar(int);

public slots:
    void sendMessage(const QByteArray &msg);
    void emergencyStop();
    void stop();
    void pause();
    void unpause();
    void step();

private:
	QSerialPort *rs232;
    SimpleTextEdit *history;
    QScrollBar *scroll_bar;
    QLineEdit *cmd;
    QQueue<QByteArray> command_queue;
    int ack_expected_before_sending_a_command;
    QByteArray last_cmd;
    QTimer *ack_timer;
    bool b_paused;
    QList<QByteArray> command_history;

    double extruder_temperature;
    double bed_temperature;
    int nb_processed_commands;
    QTime start_time;

private:
    static SerialConsole *pInstance;
public:
    static SerialConsole *instance();
};

#endif // SERIALCONSOLE_H
