#ifndef FILEWRITER_H
#define FILEWRITER_H

#include <string>
#include <cstring>
#include <cmath>
#include "utils.h"
#include <vector>

class StreamWriter
{
public:
	StreamWriter(const size_t capacity = 0);
	~StreamWriter();

	inline const char *data() const	{	return _data.data();	}
	inline size_t size() const	{	return _data.size();	}

	inline void write(const void * const data, const size_t len)
	{
		_data.reserve(_data.size() + len);
		_data.insert(_data.end(), (const char*)data, ((const char*)data) + len);
	}

	template<typename T>
	inline void write(const T &v)
	{
		write(&v, sizeof(T));
	}

	inline void write(const char &c)
	{
		_data.push_back(c);
	}

	inline void write(const std::string &str)
	{
		write(str.data(), str.size());
	}

	inline StreamWriter &operator<<(const std::string &str)
	{
		write(str);
		return *this;
	}

	inline StreamWriter &operator<<(const char *str)
	{
		write(str, strlen(str));
		return *this;
	}

	inline StreamWriter &operator<<(char c)
	{
		write(c);
		return *this;
	}

	inline StreamWriter &operator<<(int i)
	{
		if (i == 0)
		{
			write('0');
			return *this;
		}
		char buf[10];
		if (i < 0)
		{
			i = -i;
			write('-');
		}
		const int w = 1 + ilog10(i);
		for(int j = w - 1 ; j >= 0 ; --j)
		{
			const int o = i;
			i /= 10;
			buf[j] = '0' + (o - i * 10);
		}
		write(buf, w);
		return *this;
	}

	inline StreamWriter &operator<<(unsigned int i)
	{
		if (i == 0)
		{
			write('0');
			return *this;
		}
		char buf[10];
		const int w = 1 + ilog10(i);
		for(int j = w - 1 ; j >= 0 ; --j)
		{
			const unsigned int o = i;
			i /= 10U;
			buf[j] = '0' + (o - i * 10U);
		}
		write(buf, w);
		return *this;
	}

	inline StreamWriter &operator<<(float f)
	{
		if (f == 0.f)
			return *this << '0';
		if (f < 0.f)
		{
			f = -f;
			write('-');
		}
		*this << int(f);
		const int d = int((f - int(f)) * 100.f);
		if (d > 0)
		{
			*this << '.';
			if (d < 10)
				*this << '0';
			*this << d;
		}
/*		const int e = ilog10d(f);
		unsigned int uf;
		if (e >= 0 && e < 6 && (uf = (unsigned int)f) == f)
			return *this << uf;
		double d = double(f) * ipow10d(-e);
		const int i = (int)d;
		write(char('0' + i));
		d -= i;
		if (d >= 1e-5)
		{
			write('.');
			for(size_t j = 0 ; j < 5 ; ++j)
			{
				d *= 10.0;
				const int id = (int)d;
				write(char('0' + id));
				d -= id;
			}
		}
		if (e != 0)
			*this << 'e' << e;*/
		return *this;
	}

	inline StreamWriter &operator<<(double f)
	{
		return *this << float(f);
	}

private:
	std::vector<char> _data;
};

#endif // FILEWRITER_H
