#ifndef VIEWER_H
#define VIEWER_H

#include <QGLWidget>
#include "vec.h"
#include "matrix.h"

class Mesh;

class Viewer : public QGLWidget
{
    Q_OBJECT
public:
    explicit Viewer(const QGLFormat &fmt);
    ~Viewer();
    
    Mesh *getMesh() {   return mesh;    }
	std::pair<Vec3, Vec3> getBuildRange() const;

signals:
    
public slots:
    void setMesh(Mesh *mesh);
    void setBuildRange(const Vec3 &lower, const Vec3 &upper);
	void transformMesh(const Mat44 &m);
	void invertNormals();

protected:
    virtual void initializeGL();
    virtual void resizeGL(int w, int h);
    virtual void paintGL();
    virtual void mousePressEvent(QMouseEvent *);
    virtual void mouseMoveEvent(QMouseEvent *);
    virtual void wheelEvent(QWheelEvent *);

private:
    void drawBox(const Vec3 &lower, const Vec3 &upper) const;

private:
    Mesh *mesh;
    Vec3 lower, upper;
    Mat44 model_matrix;
    std::pair<Vec3, Vec3> bbox;
    QPoint last_cursor_pos;

private:
    static Viewer *pInstance;
public:
    static Viewer *instance();
};

#endif // VIEWER_H
